"""Views for the file_validator app."""
import csv
import io
from typing import Any

from django.http import HttpRequest, HttpResponse, JsonResponse
from django.shortcuts import render
from django.urls import reverse_lazy
from django.utils import timezone
from django.views.generic.base import View
from django.views.generic.edit import FormView

from sage_validation.file_validator.forms import CSVUploadForm
from sage_validation.file_validator.models import MeoCostCentres, XxData


def index_view(request: HttpRequest) -> HttpResponse:
    """Render the index page."""
    return render(request, "index.html")


class CSVUploadView(FormView):
    """View for uploading a CSV file."""

    template_name = "upload.html"
    form_class = CSVUploadForm
    success_url = reverse_lazy("upload-file")

    def get_context_data(self, **kwargs: dict[str, Any]) -> dict[str, Any]:
        """Render the form with no error message on GET request."""
        context = super().get_context_data(**kwargs)
        context["error"] = None
        context["message"] = None
        return context

    def form_valid(self, form: CSVUploadForm) -> JsonResponse:
        """Handle the CSV validation, store valid data, and prepare for export."""
        csv_file = form.cleaned_data["file"]
        csv_file.seek(0)
        decoded_file = csv_file.read().decode("utf-8").strip()

        if not decoded_file:
            return JsonResponse({"status": "error", "message": "Uploaded file is empty."}, status=400)

        reader = csv.DictReader(io.StringIO(decoded_file))
        csv_data: list[dict[str, str]] = list(reader)

        updated_data = self.update_fields(csv_data)

        self.request.session["validated_csv"] = updated_data
        self.request.session.modified = True

        return JsonResponse({
            "status": "success",
            "message": "File successfully uploaded and processed.",
            "download_url": reverse_lazy("export-file")
        })

    def form_invalid(self, form: CSVUploadForm) -> JsonResponse:
        """Handle the form when it is invalid."""
        return JsonResponse({"status": "error", "errors": form.errors}, status=400)

    @staticmethod
    def update_fields(csv_data: list[dict[str, str]]) -> list[dict[str, str]]:
        """Automatically update specific fields before export."""
        current_date: str = timezone.now().strftime("%d/%m/%Y")

        xx_data_map: dict[str, tuple] = {
            obj.xx_value: (obj.project, obj.overhead) for obj in XxData.objects.using("meo").all()
        }
        cost_centre_map: dict[str, str] = {
            obj.cc: obj.cc_type for obj in MeoCostCentres.objects.using("meo").all()
        }

        for row in csv_data:
            row["TransactionDate"] = current_date

            repeat = 1
            while f"NominalAnalysisNominalCostCentre/{repeat}" in row:
                cc = row.get(f"NominalAnalysisNominalCostCentre/{repeat}", "")
                nominal_account_name = row.get(f"NominalAnalysisNominalAccountNumber/{repeat}", "")

                if not cc or not nominal_account_name:
                    repeat += 1
                    continue

                cc_type = cost_centre_map.get(cc, "")
                xx_data = xx_data_map.get(nominal_account_name)

                if xx_data:
                    row[f"NominalAnalysisNominalAccountNumber/{repeat}"] = (
                        xx_data[0] if cc_type == "Project" else xx_data[1]
                    )

                repeat += 1

        return csv_data


class CSVExportView(View):
    """View for exporting the updated CSV file."""

    def get(self, request: HttpRequest) -> HttpResponse:
        """Generate a downloadable CSV file with updated values."""
        csv_data: list[dict[str, str]] = request.session.get("validated_csv", [])

        if not csv_data:
            return HttpResponse("No data available for export.", status=400)

        response = HttpResponse(content_type="text/csv")
        response["Content-Disposition"] = 'attachment; filename="updated_file.csv"'

        writer = csv.DictWriter(response, fieldnames=csv_data[0].keys())
        writer.writeheader()
        writer.writerows(csv_data)

        return response
